const puppeteer = require('puppeteer');

const config =
  process.env.NODE_ENV === 'development'
    ? {
        headless: false,
        slowMo: 80,
        devtools: true
      }
    : {};

const timeout = 10000;

describe('<TagInputControl />', () => {
  let page;
  let browser;

  beforeAll(async () => {
    browser = await puppeteer.launch(config);
    page = await browser.newPage();
    page.setViewport({
      width: 1080,
      height: 1080
    });
    await page.goto('http://localhost:8080');
  });

  it(
    'init tag control',
    async () => {
      const tagEls = await page.$$('.tagi__element');
      expect(tagEls.length).toBe(1);
      expect(await page.$eval('.tagi__element span', (el) => el.textContent)).toBe('initial tag');
    },
    timeout
  );

  it(
    'enter a tag',
    async () => {
      await page.click('.tagi__input');
      await page.type('.tagi__input', 'abc');
      await page.keyboard.press('Enter');
      const tagEls = await page.$$('.tagi__element');
      expect(tagEls.length).toBe(2);
      expect(await page.$$eval('.tagi__element', (el) => el[1].querySelector('span').textContent)).toBe('abc');
    },
    timeout
  );

  it(
    'enter many tags',
    async () => {
      await page.click('.tagi__input');
      for (let i = 0; i < 4; i++) {
        await page.type('.tagi__input', `abc ${i}`, { delay: 80 });
        await page.keyboard.press('Enter');
      }
      const tagEls = await page.$$('.tagi__element');
      expect(tagEls.length).toBe(6);
    },
    timeout
  );

  it(
    'delete a tag by clicking on delete icon',
    async () => {
      const rect = await page.evaluate(() => {
        const result = document.querySelector('.tagi__element svg').getBoundingClientRect();
        return {
          x: result.x,
          y: result.y
        };
      });
      await page.mouse.click(rect.x + 2, rect.y + 2, { deplay: 80 });
      const tagEls = await page.$$('.tagi__element');
      expect(tagEls.length).toBe(5);
    },
    timeout
  );

  it(
    'delete a tag by hiting backspace',
    async () => {
      await page.focus('.tagi__input');
      await page.keyboard.press('Backspace');
      expect(
        await page.evaluate(() => {
          const spanEl = document.activeElement.querySelector('span');
          return spanEl ? spanEl.textContent : '';
        })
      ).toBe('abc 3');
      await page.keyboard.press('Backspace');
      expect(
        await page.evaluate(() => {
          const spanEl = document.activeElement.querySelector('span');
          return spanEl ? spanEl.textContent : '';
        })
      ).toBe('abc 2');
      expect((await page.$$('.tagi__element')).length).toBe(4);
    },
    timeout
  );

  it(
    'edit a tag and then press enter',
    async () => {
      await page.click('.tagi__element span');
      expect(await page.$eval('.tagi__input', (el) => el.value)).toBe('abc');
      await page.type('.tagi__input', ' edited');
      await page.keyboard.press('Enter', { delay: 80 });
      expect(await page.$eval('.tagi__element span', (el) => el.textContent)).toBe('abc edited');
    },
    timeout
  );

  it(
    'edit a tag and then press esc',
    async () => {
      await page.click('.tagi__element span');
      expect(await page.$eval('.tagi__input', (el) => el.value)).toBe('abc edited');
      await page.type('.tagi__input', ' esc');
      await page.keyboard.press('Escape');
      expect(await page.$eval('.tagi__input', (el) => el.value)).toBe('');
      expect(await page.$eval('.tagi__element span', (el) => el.textContent)).toBe('abc edited esc');
    },
    timeout
  );

  it(
    'get list of tags by onTagChange callback function',
    async () => {
      await page.click('.tagi__input');
      await page.type('.tagi__input', 'the last');
      await page.keyboard.press('Enter');
      const rect = await page.evaluate(() => {
        const result = document.querySelector('.tagi__element svg').getBoundingClientRect();
        return {
          x: result.x,
          y: result.y
        };
      });
      await page.mouse.click(rect.x + 2, rect.y + 2, { deplay: 80 });
      const removedtags = await page.evaluate(() => window.lastestTagList);
      expect(removedtags).toEqual(expect.arrayContaining(['abc 0', 'abc 1', 'abc 2', 'the last']));
    },
    timeout
  );

  afterAll(() => {
    browser.close();
  });
});
